package ij.plugin.filter;
import ij.*;
import ij.gui.*;
import ij.process.*;
import ij.measure.*;
import java.awt.*;

/** Implements the Analyze/Calibrate command. */
public class Calibrator implements PlugInFilter {

	private static final String NONE = "None";
	private static final String INVERTER = "Pixel Inverter";
	private static final String UNCALIBRATED_OD = "Uncalibrated OD";
	private static final String[] functions = {NONE,INVERTER,UNCALIBRATED_OD};
	static boolean global;
	private static boolean oldGlobal;
    private ImagePlus imp;
	private String choice;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		IJ.register(Calibrator.class);
		return DOES_8G+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		oldGlobal = global;
		if (!showDialog(imp))
			return;
		calibrate(imp);
		if (global || global!=oldGlobal) {
			int[] list = WindowManager.getIDList();
			if (list==null)
				return;
			for (int i=0; i<list.length; i++) {
				ImagePlus imp2 = WindowManager.getImage(list[i]);
				if (imp2!=null)
					imp2.getWindow().repaint();
			}
		} else
			imp.getWindow().repaint();
	}

	public boolean showDialog(ImagePlus imp) {
		String defaultChoice;
		Calibration cal = imp.getCalibration();
		int function = cal.getFunction();
		switch (function) {
			case Calibration.NONE: defaultChoice=NONE; break;
			case Calibration.INVERTER: defaultChoice=INVERTER; break;
			case Calibration.UNCALIBRATED_OD: defaultChoice=UNCALIBRATED_OD; break;
			default: defaultChoice=NONE;
		}
		GenericDialog gd = new GenericDialog("Calibrate...", IJ.getInstance());
		gd.addChoice("Function:", functions, defaultChoice);
		gd.addCheckbox("Global", global);
		gd.showDialog();
		if (gd.wasCanceled())
			return false;
		else {
			choice = gd.getNextChoice();
			global = gd.getNextBoolean();
			return true;
		}
	}

	public void calibrate(ImagePlus imp) {
		Calibration cal = imp.getCalibration();
		int function = Calibration.NONE;
		float[] table = null;
		if (choice.equals(INVERTER)) {
			table = new float[256];
			for (int i=0; i<256; i++)
				table[i] = 255-i;
			function = Calibration.INVERTER;
		} else if (choice.equals(UNCALIBRATED_OD)) {
			table = new float[256];
			boolean invertedLut = imp.isInvertedLut();
			for (int i=0; i<256; i++)
				table[i] = (float)od(i,invertedLut);
			function = Calibration.UNCALIBRATED_OD;
		}
		cal.setCTable(table, function);
		if (global)
			imp.setGlobalCalibration(cal);
		else
			imp.setCalibration(cal);
		//IJ.write("cal: "+cal);
	}

	double od(double v, boolean invertedLut) {
		if (invertedLut) {
			if (v==255.0) v = 254.5;
			return 0.434294481*Math.log(255.0/(255.0-v));
		} else {
			if (v==0.0) v = 0.5;
			return 0.434294481*Math.log(255.0/v);
		}
	}

}